//封装 HTTP 请求，提供统一的错误处理和请求配置
import Taro from '@tarojs/taro'

// 接口响应类型
interface ApiResponse<T = any> {
  code: number
  data: T
  message: string
}

// 请求选项
interface RequestOptions {
  showLoading?: boolean
  loadingText?: string
  showError?: boolean
}

// 默认选项
const defaultOptions: RequestOptions = {
  showLoading: true,
  loadingText: '加载中...',
  showError: true
}

/**
 * 请求封装
 * @param url 请求地址
 * @param method 请求方法
 * @param data 请求数据
 * @param options 请求选项
 */
export async function request<T = any>(
  url: string,
  method: keyof Taro.request.Method = 'GET',
  data?: any,
  options?: RequestOptions
): Promise<T> {
  const opt = { ...defaultOptions, ...options }
  
  // 显示加载中
  if (opt.showLoading) {
    Taro.showLoading({
      title: opt.loadingText || '加载中...' // 添加默认值，确保不会是undefined
    })
  }
  
  try {
    const response = await Taro.request<ApiResponse<T>>({
      url,
      method,
      data,
      header: {
        'content-type': 'application/json'
      }
    })
    
    // 隐藏加载中
    if (opt.showLoading) {
      Taro.hideLoading()
    }
    
    // 接口请求错误
    if (response.statusCode !== 200) {
      if (opt.showError) {
        Taro.showToast({
          title: `请求错误: ${response.statusCode}`,
          icon: 'none'
        })
      }
      throw new Error(`请求错误: ${response.statusCode}`)
    }
    
    // 业务错误
    if (response.data.code !== 0) {
      if (opt.showError) {
        Taro.showToast({
          title: response.data.message || '请求失败', // 添加默认值
          icon: 'none'
        })
      }
      throw new Error(response.data.message || '请求失败') // 添加默认值
    }
    
    return response.data.data
  } catch (error: any) { // 指定error类型为any，以便访问message属性
    // 隐藏加载中
    if (opt.showLoading) {
      Taro.hideLoading()
    }
    
    // 显示错误
    if (opt.showError) {
      Taro.showToast({
        title: error.message || '请求异常', // 添加默认值
        icon: 'none'
      })
    }
    
    throw error
  }
}