// services/room.ts
import Taro from '@tarojs/taro'
import { 
  type RecentRoom, 
  type Room, 
  type RoomUserInfo, 
  RoomRole, 
  RoomStatus, 
  RoomVisibility 
} from '@/types/room'
import { type UserInfo } from '@/types/user'

// Mock房间数据（测试用）
const mockRooms: Record<string, Room> = {
  'room_001': {
    id: 'room_001',
    name: '海龟汤房间',
    gameId: '1',
    gameTitle: '海龟汤',
    maxPlayers: 10,
    visibility: RoomVisibility.PUBLIC,
    hosterId: 'host_123',
    createTime: Date.now() - 3600000,
    status: RoomStatus.WAITING,
    users: [
      {
        openid: 'host_123',
        nickname: '主持人',
        avatar: '',
        roomRole: RoomRole.HOSTER,
        joinTime: Date.now() - 3600000,
        isReady: true
      }
    ]
  },
  'room_002': {
    id: 'room_002',
    name: '狼人杀小队',
    gameId: '2',
    gameTitle: '狼人杀',
    maxPlayers: 8,
    visibility: RoomVisibility.PRIVATE,
    password: '1234',
    hosterId: 'host_456',
    createTime: Date.now() - 86400000,
    status: RoomStatus.WAITING,
    users: [
      {
        openid: 'host_456',
        nickname: '狼王',
        avatar: '',
        roomRole: RoomRole.HOSTER,
        joinTime: Date.now() - 86400000,
        isReady: true
      }
    ]
  }
};

// 房间API - 适配新数据结构
export const roomService = {
  // 获取最近房间
  async getRecentRooms(): Promise<RecentRoom[]> {
    try {
      // 从缓存获取
      const cachedRooms = Taro.getStorageSync('recentRooms')
      if (cachedRooms) {
        return JSON.parse(cachedRooms)
      }
      
      // 返回默认mock数据
      return [
        { 
          id: 'room_001', 
          name: '海龟汤房间', 
          gameTitle: '海龟汤', 
          lastVisitTime: Date.now() - 3600000, 
          hasPassword: false, 
          status: RoomStatus.WAITING,
          playerCount: 1
        },
        { 
          id: 'room_002', 
          name: '狼人杀小队', 
          gameTitle: '狼人杀', 
          lastVisitTime: Date.now() - 86400000, 
          hasPassword: true, 
          status: RoomStatus.PLAYING,
          playerCount: 1
        }
      ]
    } catch (error) {
      console.error('获取最近房间失败:', error)
      return []
    }
  },
  
  // 检查房间是否存在
  async checkRoomExist(roomId: string): Promise<{exists: boolean, needPassword: boolean}> {
    try {
      // 先查询本地存储
      const recentRooms = Taro.getStorageSync('recentRooms')
      if (recentRooms) {
        const rooms = JSON.parse(recentRooms)
        const room = rooms.find((r: RecentRoom) => r.id === roomId)
        if (room) {
          return { exists: true, needPassword: room.hasPassword }
        }
      }
      
      // 如果本地没有，检查mock数据
      const mockRoom = mockRooms[roomId]
      if (mockRoom) {
        return { 
          exists: true, 
          needPassword: mockRoom.visibility === RoomVisibility.PRIVATE 
        }
      }
      
      return { exists: false, needPassword: false }
    } catch (error) {
      console.error('检查房间失败:', error)
      return { exists: false, needPassword: false }
    }
  },
  
  // 验证房间密码
  async verifyRoomPassword(roomId: string, password: string): Promise<{valid: boolean}> {
    try {
      // 简化实现：仅检查mock数据中的密码
      const mockRoom = mockRooms[roomId]
      if (mockRoom && mockRoom.visibility === RoomVisibility.PRIVATE) {
        return { valid: mockRoom.password === password }
      }
      
      return { valid: true }
    } catch (error) {
      console.error('验证房间密码失败:', error)
      return { valid: false }
    }
  },
  
  // 创建新房间 - 适配新数据结构
  async createRoom(roomData: Room, userInfo: UserInfo): Promise<{roomId: string | null, success: boolean, message?: string}> {
    try {
      if (!userInfo.openid) {
        return { roomId: null, success: false, message: '用户未登录' }
      }
      
      // 本地存储一份房间信息
      const recentRooms = Taro.getStorageSync('recentRooms') || '[]'
      const rooms = JSON.parse(recentRooms)
      
      // 添加到最近房间
      const roomInfo: RecentRoom = {
        id: roomData.id,
        name: roomData.name,
        gameTitle: roomData.gameTitle,
        lastVisitTime: roomData.createTime,
        hasPassword: roomData.visibility === RoomVisibility.PRIVATE,
        status: roomData.status,
        playerCount: roomData.users.length
      }
      
      rooms.unshift(roomInfo)
      // 只保留最近10个房间
      if (rooms.length > 10) {
        rooms.pop()
      }
      
      Taro.setStorageSync('recentRooms', JSON.stringify(rooms))
      
      // 简化实现：仅存储在本地缓存
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      roomsObj[roomData.id] = roomData
      Taro.setStorageSync('allRooms', JSON.stringify(roomsObj))
      
      return { roomId: roomData.id, success: true }
    } catch (error) {
      console.error('创建房间失败:', error)
      return { roomId: null, success: false, message: '创建房间失败' }
    }
  },
  
  // 加入房间 - 适配新数据结构
  async joinRoom(roomId: string, userData: RoomUserInfo): Promise<{success: boolean, roomData?: Room, message?: string}> {
    try {
      // 查找房间 - 先从本地缓存查询
      let room: Room | null = null
      
      // 检查本地缓存中是否有该房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      if (roomsObj[roomId]) {
        room = roomsObj[roomId]
      } else if (mockRooms[roomId]) {
        // 使用mock数据
        room = mockRooms[roomId]
      }
      
      if (!room) {
        return { success: false, message: '房间不存在' }
      }
      
      // 检查房间是否已满
      if (room.users && room.users.length >= room.maxPlayers) {
        return { success: false, message: '房间已满' }
      }
      
      // 检查用户是否已在房间中
      const existingUser = room.users.find(u => u.openid === userData.openid)
      if (!existingUser) {
        // 添加用户到房间
        room.users.push(userData)
        
        // 更新本地缓存
        roomsObj[roomId] = room
        Taro.setStorageSync('allRooms', JSON.stringify(roomsObj))
      }
      
      // 保存到最近加入的房间
      const roomInfo: RecentRoom = {
        id: room.id,
        name: room.name,
        gameTitle: room.gameTitle,
        lastVisitTime: Date.now(),
        hasPassword: room.visibility === RoomVisibility.PRIVATE,
        status: room.status,
        playerCount: room.users.length
      }
      
      // 更新最近房间列表
      this.saveToRecentRooms(roomInfo)
      
      return { success: true, roomData: room }
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  },
  
  // 保存到最近加入的房间
  async saveToRecentRooms(roomInfo: RecentRoom): Promise<void> {
    try {
      const recentRooms = Taro.getStorageSync('recentRooms') || '[]'
      const rooms = JSON.parse(recentRooms)
      
      // 检查是否已存在
      const existingIndex = rooms.findIndex((r: RecentRoom) => r.id === roomInfo.id)
      if (existingIndex !== -1) {
        // 移除已存在的
        rooms.splice(existingIndex, 1)
      }
      
      // 添加到最前面
      rooms.unshift(roomInfo)
      
      // 只保留最近10个房间
      if (rooms.length > 10) {
        rooms.pop()
      }
      
      Taro.setStorageSync('recentRooms', JSON.stringify(rooms))
    } catch (error) {
      console.error('保存最近房间失败:', error)
    }
  },
  
  // 退出房间
  async leaveRoom(roomId: string, userId: string): Promise<{success: boolean, message?: string}> {
    try {
      // 从本地缓存找到房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      if (roomsObj[roomId]) {
        const room = roomsObj[roomId]
        
        // 过滤掉当前用户
        room.users = room.users.filter(u => u.openid !== userId)
        
        // 更新本地缓存
        roomsObj[roomId] = room
        Taro.setStorageSync('allRooms', JSON.stringify(roomsObj))
      }
      
      return { success: true }
    } catch (error) {
      console.error('退出房间失败:', error)
      return { success: false, message: '退出房间失败' }
    }
  },
  
  // 获取房间信息
  async getRoomInfo(roomId: string): Promise<Room | null> {
    try {
      // 检查本地缓存中是否有该房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      if (roomsObj[roomId]) {
        return roomsObj[roomId]
      }
      
      // 如果本地没有，使用mock数据
      if (mockRooms[roomId]) {
        return mockRooms[roomId]
      }
      
      return null
    } catch (error) {
      console.error('获取房间信息失败:', error)
      return null
    }
  },

  // 获取进行中的房间
  async getActiveRooms(userId: string): Promise<Room[]> {
    try {
      // 从缓存或模拟数据获取所有房间
      const allRooms = Taro.getStorageSync('allRooms') || '{}'
      const roomsObj = JSON.parse(allRooms)
      
      // 添加类型断言，确保 Object.values 返回的是 Room[] 类型
      const activeRooms = Object.values(roomsObj) as Room[];
      
      // 过滤出用户参与的且状态为"waiting"或"playing"的房间
      const userActiveRooms = activeRooms.filter(room => {
        return room.status !== RoomStatus.ENDED && 
          room.users && 
          room.users.some(u => u.openid === userId)
      })
      
      // 添加mock数据用于测试
      if (userActiveRooms.length === 0) {
        return [
          { 
            id: 'room_001', 
            name: '欢乐海龟汤', 
            gameId: '1', 
            gameTitle: '海龟汤',
            hosterId: 'host_123',
            status: RoomStatus.PLAYING,
            users: [
              {
                openid: userId,
                nickname: '玩家',
                avatar: '',
                roomRole: RoomRole.PLAYER,
                joinTime: Date.now(),
                isReady: true
              }
            ],
            createTime: Date.now() - 3600000,
            maxPlayers: 10,
            visibility: RoomVisibility.PUBLIC
          },
          { 
            id: 'room_002', 
            name: '趣味谜题', 
            gameId: '1', 
            gameTitle: '海龟汤',
            hosterId: userId,
            status: RoomStatus.WAITING,
            users: [
              {
                openid: userId,
                nickname: '玩家',
                avatar: '',
                roomRole: RoomRole.HOSTER,
                joinTime: Date.now(),
                isReady: true
              }
            ],
            createTime: Date.now() - 7200000,
            maxPlayers: 8,
            visibility: RoomVisibility.PUBLIC
          }
        ]
      }
      
      return userActiveRooms
    } catch (error) {
      console.error('获取进行中房间失败:', error)
      return []
    }
  },

  // 获取已结束的游戏
  async getEndedGames(userId: string): Promise<any[]> {
    try {
      // 从缓存获取
      const endedGames = Taro.getStorageSync('endedGames') || '[]'
      const games = JSON.parse(endedGames)
      
      // 筛选用户参与的游戏
      const userGames = games.filter((game: any) => 
        game.users && game.users.some((u: any) => u.openid === userId)
      )
      
      // 添加mock数据用于测试
      if (userGames.length === 0) {
        return [
          { 
            id: 'game_001', 
            roomId: 'room_003',
            roomName: '神秘的手表',
            gameId: '1', 
            gameTitle: '海龟汤', 
            hosterId: 'host_123',
            hosterName: '小李',
            endTime: Date.now() - 86400000, // 昨天
            duration: 45, // 分钟
            users: [{ openid: userId }]
          },
          { 
            id: 'game_002', 
            roomId: 'room_004',
            roomName: '迷路的青蛙',
            gameId: '1', 
            gameTitle: '海龟汤', 
            hosterId: userId,
            hosterName: '玩家',
            endTime: Date.now() - 172800000, // 前天
            duration: 28, // 分钟
            users: [{ openid: userId }]
          },
          { 
            id: 'game_003', 
            roomId: 'room_005',
            roomName: '动物园奇案',
            gameId: '2', 
            gameTitle: '谜星探案', 
            hosterId: 'host_456',
            hosterName: '探长',
            endTime: Date.now() - 259200000, // 3天前
            duration: 35, // 分钟
            users: [{ openid: userId }]
          }
        ]
      }
      
      return userGames
    } catch (error) {
      console.error('获取已结束游戏失败:', error)
      return []
    }
  },

  // 获取我创建的房间/游戏
  async getCreatedRooms(userId: string): Promise<any[]> {
    try {
      // 合并进行中和已结束的，筛选出我创建的
      const activeRooms = await this.getActiveRooms(userId)
      const endedGames = await this.getEndedGames(userId)
      
      const createdRooms = activeRooms.filter((room: Room) => room.hosterId === userId)
      const createdGames = endedGames.filter((game: any) => game.hosterId === userId)
      
      // 合并结果并按时间排序
      const result = [
        ...createdRooms.map((room: Room) => ({
          ...room,
          isActive: true,
          time: room.createTime
        })),
        ...createdGames.map((game: any) => ({
          ...game,
          isActive: false,
          time: game.endTime
        }))
      ].sort((a, b) => b.time - a.time)
      
      return result
    } catch (error) {
      console.error('获取我创建的房间/游戏失败:', error)
      return []
    }
  },

  // 获取我参与的房间/游戏
  async getJoinedRooms(userId: string): Promise<any[]> {
    try {
      // 合并进行中和已结束的，筛选出我参与但不是我创建的
      const activeRooms = await this.getActiveRooms(userId)
      const endedGames = await this.getEndedGames(userId)
      
      const joinedRooms = activeRooms.filter((room: Room) => 
        room.hosterId !== userId &&
        room.users && 
        room.users.some((u) => u.openid === userId)
      )
      
      const joinedGames = endedGames.filter((game: any) => 
        game.hosterId !== userId &&
        game.users && 
        game.users.some((u: any) => u.openid === userId)
      )
      
      // 合并结果并按时间排序
      const result = [
        ...joinedRooms.map((room: Room) => ({
          ...room,
          isActive: true,
          time: room.createTime
        })),
        ...joinedGames.map((game: any) => ({
          ...game,
          isActive: false,
          time: game.endTime
        }))
      ].sort((a, b) => b.time - a.time)
      
      return result
    } catch (error) {
      console.error('获取我参与的房间/游戏失败:', error)
      return []
    }
  },

  // 创建房间用户信息
  createRoomUserInfo(userInfo: UserInfo, isHoster: boolean = false): RoomUserInfo {
    return {
      ...userInfo,
      roomRole: isHoster ? RoomRole.HOSTER : RoomRole.PLAYER,
      joinTime: Date.now(),
      isReady: isHoster
    }
  }
}